import string, re, os,random
from Tree import *
from usefull import *

class Matrix(object):
    def __init__(self,infile="",taxdirfile="",matrix={},name=""): 
        if len(infile)>0:                          #initialization with file
            data=self.__parse_infile(infile)       
            self.name=infile
        else:                   #initialization with data matrix 
            data=matrix
            if len(name)>0:self.name=name
            else: name="matrix"

        self.data=data
        self.alglength=len(self.data.values()[0])

    def __parse_infile(self,infile):
        print "parsing file %s" %infile
        inf= open(infile)
        if re.compile("#nexus", re.IGNORECASE).search(inf.readline())!=None:format='nexus'
        else: format='phylip'
        matrix={}

        if format=='nexus':
            mat=False
            comment=re.compile("^[\s]*\[")
            p=re.compile('^[^\[]*matrix',re.IGNORECASE)
            for line in inf.readlines():
                if not mat:
                    if p.search(line) != None: mat=True
                    continue
                if line.find(';')!=-1: break
                if comment.search(line)!=None:continue
                list=line.split()
                if len(list)==0:continue
                acc=list[0]
                seq=""                         #Nexus file may contain spaces in sequence line
                for j in range(1,len(list)):
                    block=list[j]
                    if block.startswith("["):break
                    seq+=block
                if matrix.has_key(acc):matrix[acc]+=seq
                else: matrix[acc]=seq

        else: #phylip
            for line in inf.readlines():
                 list=line.split()
                 acc=list[0]
                 seq=list[1]
                 if matrix.has_key(acc):matrix[acc]+=seq
                 else: matrix[acc]=seq
                     
        print "finished parsing"
        return matrix

    def save(self,outfile="",format='phylip',ext=True): #format nexus|phylip
        if len(outfile)>0:fname=outfile
        else:fname=self.name
        taxonorder=self.data.keys()
        taxonorder.sort()

        if format == 'nexus':
            if ext:fname+='.nex'
            outf=open(fname,'w')
            outf.write("#Nexus\nBegin data;dimensions ntax=%u nchar=%u;\nformat datatype=protein gap=- missing=?;matrix\n" %(len(self.data),len(self.data[self.data.keys()[0]])))
            for staxon in taxonorder:
                name=staxon
                outf.write("%s %s\n"%(name,self.data[staxon]))
            outf.write("\n;end;\n")

            
        elif format=='phylip':
            if ext:fname+='.phy'
            outf=open(fname,'w')
            outf.write(" %u %u\n" %(len(self.data),self.alglength))
            for staxon in taxonorder:
                name=staxon
                outf.write("%s %s%s\n"%(name,' '*(9-len(name)),self.data[staxon]))

    def get_matrix(self):
        return self.data

    def reconstruct_tree(self,filename="",bootstrap=0,root="-",iqpnni=""): #if no name given, safe matrix before computing, otherwise, take file, 
        if len(filename)==0:name=self.name+".phy"
        else: name=filename+".phy"
        self.__call_ipqnni(name,root,iqpnni)
        if bootstrap>0:
            seed=random.randint(0,8191)*4+1
            print "compute bootstrapping trees"
            open('parfile','w').write("%s\nR\n%u\nY\n%u\n" %(name,bootstrap,seed))
            os.system('rm outfile')
            command('seqboot <parfile >boot.out')
            self.__parse_bootstrap('outfile',name)
            os.system('rm parfile outfile boot.out')
            for i in range (1,bootstrap+1):
                lname=name+str(i)
                self.__call_ipqnni(lname,root,iqpnni)
                os.system("rm -f %s%u.iqpnni.bionj %s%u.iqpnni.dist %s%u.iqpnni"% tuple([name,i]*3))
            print "compute consensus"
            os.system('cat %s.phy[0-9]*.iqpnni.treefile > %s.trees' %(self.name,self.name))
            os.system('rm -f outfile outtree intree  %s.phy[0-9]*.iqpnni.treefile' %self.name)
            
            open('cons.in','w').write("%s.trees\nC\nC\nY\n" %self.name) #unrooted
            
            os.system('consense <cons.in >cons.out') #majority rule
            os.system('mv outtree %s.trees.cons' %self.name)
            tname=self.name+'.trees.cons'
        else:tname=name+'.iqpnni.treefile'
            
        mytree=Tree(infile=tname)
        mytree.write_tree(name=self.name+".source")        
        
    def __call_ipqnni(self,name,root,iqpnni):
        cmd="iqpnni %s -ni %s >%s.iqpnni.out" %(iqpnni,name,name)
        command(cmd)
        os.system("rm -f %s.iqpnni.prediction %s.iqpnni.treels %s.iqpnni.checkpoint %s.iqpnni.out %s.iqpnni.rate" % tuple([name]*5))
        
    def __parse_bootstrap(self,outfile,name):
        i=1
        newf=re.compile("^\s*\d+\s+\d+\s*$")
        for line in open(outfile).readlines():
            if newf.match(line):
                outf=open("%s%u"%(name,i),'w')
                i+=1
            outf.write(line)
        
    def prune(self,taxa):
        if type(taxa)==dict:taxa=taxa.keys()
        newdata=self.data.copy()
        for t in newdata.keys():
            if t not in taxa:del newdata[t]
        return Matrix(matrix=newdata,name=self.name,save_taxa=False)

class Supermatrix(Matrix):
    
    def __init__(self,infile="",taxdirfile="",matrix={},taxdir={},name="",root="-"):  #root -  means no explicit root
        Matrix.__init__(self,infile,taxdirfile,matrix,name)
        self.matrices=[]
        self.root=root

    def create_org_matrices(self,number=0,sections=""):
        if len(sections)==0:
            self.number=number
            print "creating original matrices from pattern of '?'"
            acc_list=self.data.keys()
            old_quest=[]
            start_column=0
            index=1
            self.matrices=[]
            j=1
            for column in range(0,len(self.data[acc_list[0]])):
                quest=self.__get_quest_list(column)
                if (start_column==0 and len(old_quest)==0):old_quest=quest
                if old_quest!=quest:
                    self.__add_matrix(old_quest,start_column,column,j)
                    j+=1
                    start_column=column
                    index+=1
                    old_quest=quest
            self.__add_matrix(old_quest,start_column,column+1,j)
        else:
            print "creating original matrices from given sections"
            if type(sections)==str:sec=[int(l[:-1]) for l in open(sections).readlines()]
            else: sec=sections
            self.number=len(sec)
            j=1
            i=1
            for s in sec:
                end=s+i
                self.__add_matrix([],i-1,end-1,j)
                j+=1
                i=end
                
        print "finished: %u matrices found" %len(self.matrices)


    def __add_matrix(self,quest,start_column,column,number):
        acc_list=[]
        for acc in self.data.keys():
            if acc not in quest: acc_list.append(acc)
        matrix={}
        for acc in acc_list:
            line=self.data[acc][start_column:column]
            if line=='?'*len(line) or  line=='X'*len(line):continue
            matrix[acc]=self.data[acc][start_column:column]
        self.matrices.append(Matrix(matrix=matrix,name=self.name+str_int(number,self.number)))

    #returns accession list of accs not in current tree
    def __get_quest_list(self,i):
        quest=[]
        for acc in self.data.keys():
            if self.data[acc][i]=='?':quest.append(acc)
        return quest

    def save_matrices(self,outfile="",format="nexus"): #format nexus|phylip
        if len(self.matrices)==0:self.create_org_matrices()
        max=len(self.matrices)
        print "saving matrices"
        i=1
        for m in self.matrices:
            if len(outfile)>0:m.save("%s%s" %(outfile,str_int(i,max)),format)
            else:m.save("",format)
            i+=1
        print "finished saving"
        
    def reconstruct_single_trees(self,filename="",bootstrap=0, iqpnni="",blcutoff=0): 
        print "starting tree reconstruction for single matrices"
        root=self.root
        trees=[]
        i=1
        if len(filename)==0:
            for mat in self.matrices:
                mat.reconstruct_tree(bootstrap=bootstrap,root=root,iqpnni=iqpnni)
                i+=1
        else:
            for mat in self.matrices:
                mat.reconstruct_tree("%s%u" %(filename,i),bootstrap,root,iqpnni)
                i+=1
        self.write_trees(filename,bootstrap)
        self.write_distances(filename)
        
    def reconstruct_tree(self,filename="",bootstrap=0,root="",iqpnni=""):
        if len(root)==0:root=self.root
        Matrix.reconstruct_tree(self,filename,bootstrap,root,iqpnni)

    def write_trees(self,name="",bootstrap=0):
        if len(name)==0:name=self.name

        open("tempfile","w").write("%u\n"%(len(self.matrices)))
        cmd="cat tempfile"
        for i in range(0,len(self.matrices)):
            cmd+=" %s%s.source.tree" %(name,str_int(i+1,self.number))
        cmd+=" >%s.source.trees" %name
        os.system(cmd)
        os.remove("tempfile")

    def write_distances(self,name=""):
        if len(name)==0:name=self.name
        open("tempfile","w").write("%u\n"%(len(self.matrices)))
        cmd="cat tempfile"
        for i in range(0,len(self.matrices)):
            cmd+=" %s%s.phy.iqpnni.dist" %(name,str_int(i+1,self.number))
        cmd+=" >%s.dists" %name
        os.system(cmd)
        os.remove("tempfile")
         
