import os,sys
from Matrix import *
from usefull import *

class SimDataSet(object):
   
    def __init__(self,orgname,newname,scale=1):
         self.orgname=orgname
         self.newname=newname
         infile=open(self.orgname)
         self.aa=int(scale*int(infile.readline().split()[1]))
         self.taxa=[]
         for line in infile.readlines():
             self.taxa.append(line.split()[0])

    def simulate(self,tree,param): #tree is name of treefile
        command("seq-gen %s -l%u -n1 %s >%s.sim 2>%s.out" %(param,self.aa,tree,self.newname,self.newname))

    def prune(self,tlist=[]):
        infile=open(self.newname+".sim")
        outfile=open(self.newname+".sim.prn",'w')
        if len(tlist)==0: #only one model tree with all taxa
            outfile.write(infile.readline()) #copy first line
            for line in infile.readlines():
                l=line.split()
                if l[0] in self.taxa:outfile.write(line)
                else:outfile.write(line[:10]+'?'*self.aa+'\n')
        else: #several model tree, not all taxa
            infile.readline()
            outfile.write(" %u %u\n"%(len(tlist),self.aa))
            data={}
            for line in infile.readlines():
                l=line.split()
                data[l[0]]=l[1]
            act_taxa=data.keys()
            for taxon in tlist:
                outfile.write("%s%s"%(taxon," "*(10-len(taxon))))
                if taxon in data.keys():outfile.write(data[taxon])
                else:outfile.write('?'*self.aa)
                outfile.write('\n')

class Simulation(object):
    def __init__(self,newname,number,tree,orgdata="",sections=""):
        print "Start simulation"
        self.name=newname
        self.dir=newname[:newname.rfind("/")+1]
        self.number=number #number of data sets
        self.org=orgdata
        self.sm=Supermatrix(orgdata,root="-")
        try: #gene trees already built
            open("%s%s.phy"%(orgdata,str_int(1,self.number)))
            print "single matrices present"
        except:
            self.sm.create_org_matrices(sections=sections)
            self.sm.save_matrices(outfile=orgdata,format="phylip")
        

        if type(tree)==str: #filename
            self.tree=tree
            self.treelist=[]
        else: #list of filenames
            if len(tree)!=number:
                print "Error, incorrect number of trees, either one or number of source trees"
                sys.exit(0)
            self.treelist=tree
            self.tree=""
        self.sections=sections

    def __get_parameter(self,scale):           #read lengths and taxa out of phylip files
        self.data=[] 
        for i in range (0,self.number): 
            self.data.append(SimDataSet("%s%s.phy"%(self.org,str_int(i+1,self.number)),"%s%s.phy"%(self.name,str_int(i+1,self.number)),scale))

    def __get_lengths(self):
        lengths=[] 
        for d in self.data:
            lengths.append(d.aa)
        return lengths

    def simulate(self,param,scale=[],seq_scale=1,theta=0): #param has to be file now!!!
        self.__get_parameter(seq_scale)
        paras=open(param).readlines()
        if len(paras)==1:param=paras[0][:-1]
        else:
            if len(paras)!=self.number:
                print "Error, incorrect number of simulation parameters in %s, specifiy either one or number of source trees" %param
                sys.exit(0)
            param=""
        tlist=[]
        if len(self.treelist)>0: #individual trees --> need to know complete taxa set
            trees=[]
            for d in self.data:
                trees+=d.taxa
            tlist=list(set(trees))
            tlist.sort()

        elif theta>0: #one species tree --> generate gene trees by coalscent
            print ("Generate gene trees with coalescent process")
            command("java -jar GenGTree.jar %f %s %s %s" %(theta,self.number,self.tree,self.name+".genetrees"))
            trees=open(self.name+".genetrees").readlines()
            for i in range(0,self.number):
                actname=self.name+".genetrees"+str(i)
                open(actname,'w').write(trees[i])
                self.treelist.append(actname)   

        print "Start sequence simulation"
        for i in range (0,self.number):
            if len(param)>0:act_para=param
            else:act_para=paras[i][:-1]
            if len(self.treelist)>0:act_tree=self.treelist[i]
            else:act_tree=self.tree
            if len(scale)==0:self.data[i].simulate(act_tree,act_para)
            else: self.data[i].simulate(act_tree,act_para+" -s %s"%scale[i])
            self.data[i].prune(tlist)

        if theta>0: os.system("rm %s" %string.join(self.treelist," "))

    def get_SM(self):
        taxa=0
        count=0
        data={}
        for i in range(0,self.number):
            infile=open("%s%s.phy.sim.prn"%(self.name,str_int(i+1,self.number)))
            line=infile.readline()
            if taxa==0:taxa=int(line.split()[0])
            l=int(line.split()[1])
            count+=l
            for line in infile.readlines():
                acc,seq=line.split()
                if data.has_key(acc):data[acc]+=seq
                else:data[acc]=seq
        return Supermatrix(matrix=data,name=self.name+'.prn',root="-")

    def write_SM(self,format='phylip',seq_scale=1):
        print "Write simulated supermatrix"
        mysm= self.get_SM()
        mysm.save(outfile=self.dir+"alg",format=format,ext=False)
        
        lengths=[int(seq_scale*int(i)) for i in self.sections]

        mysm.create_org_matrices(sections=lengths)
        mysm.save_matrices(outfile=self.dir+'alg',format=format)
        os.system("rm %s*.out %s*.prn"%tuple([self.name]*2))

