#!/usr/bin/env python

from Matrix import *
from Simulation import *
from usefull import *
import os,time,sys,random


def parse_options():
    from optparse import OptionParser, OptionGroup
    parser = OptionParser()
    
    group=OptionGroup(parser,"Input options")
    group.add_option("-s", "--supermatrix", dest="data", help="location of original supermatrix in phylip or nexus format, for simulation matrix.nex1.phy etc. are assumed to be in same directory or written there, default=./matrix.nex", default="./matrix.nex")
    group.add_option("-t", "--section",dest="sec",help="file with lengths of individual genes in alignment",default="")
    parser.add_option_group(group)

    group=OptionGroup(parser,"Simulation options")
    group.add_option("-o", "--offset", dest="start", help="number of first simulation, default=1",default=1)
    group.add_option("-e", "--end", dest="end",help="number of last simulation, default=1",default=1)
    group.add_option("-u", "--multsource",dest="mult",help="use gene trees for simulation, stored in 'model_tree[1-source].tree' in data directory, default: off (only 'model_tree')",action="store_true",default=False)
    group.add_option("-g", "--seqgen",dest="seqgen",help="file in simulation directory with options for seq-gen, default 'seq-gen.para' (at least substitution model needed)",default="seq-gen.para")
    group.add_option("-f", "--tree_scale",dest="scale",help="file with tree lengths scalings for sections, default no scaling",default="")
    group.add_option("-l", "--length_scale",dest="lscale",help="file with sequence lengths scalings for alignments, default 1",default=1)
    group.add_option("-v", "--theta",dest="theta",help="specify theta if gene trees should be generated from species tree by coalescent model (default: no gene trees)",default=0)
    group.add_option("-p", "--phylo",dest='phylo', help="type of phylogeny reconstruction: a - all (default), i - input trees only, s - superalignment tree only, n - none", default='a')
    group.add_option("-i", "--iqpnni",dest='iqpnni',help="command line parameters for iqpnni, default '-w gamma'", default='-w gamma')
    group.add_option("-b", "--bootstrap",dest="boot",help='number of bootstrap replicates for ipqnni (0-off, default)',default=0)
    parser.add_option_group(group)
    

    group=OptionGroup(parser,"Output options")
    group.add_option("-d", "--dir",    dest="dir",   help="simulation directory, default is current directory",default="./")
    parser.add_option_group(group)
    
    (options, args) = parser.parse_args()
    if len(options.sec)==0:
        parser.print_help()
        sys.exit(0)
    return options,args


def simulate(dir,name,i,parameter,model_tree,scale=[],seq_scale=1,theta=0,sec_list=[]):
    
    actdir="%ssim_%u/"%(dir,i)
    print "create directory %s" %actdir
    os.mkdir(actdir)

    source=len(sec_list)
    print "simulate %u datasets according to parameters of %s" %(source,name)
    mysim=Simulation(newname=actdir+'org',number=source,tree=model_tree,orgdata=name,sections=sec_list)

    mysim.simulate(parameter,scale,seq_scale,theta)
    mysim.write_SM(seq_scale=seq_scale)
    return mysim

def reconstruct(actdir,name,mysm,iqpnni,boot,phylo):
    mysm.save(format='nexus')
    mysm.save(format='phylip')

    if phylo=="n": return
    if phylo!="s": mysm.reconstruct_single_trees(iqpnni=iqpnni,bootstrap=boot)
    if phylo!="i":
        mysm.reconstruct_tree(filename=name,iqpnni=iqpnni,bootstrap=boot)
        if boot>0:open("%ssm.tree"%actdir,'w').write(string.join([i[:-1] for i in open("%s.trees.cons"%(name)).readlines()],''))
        else: os.system("cp %s.phy.iqpnni.treefile %ssm.tree" %(name,actdir))



def new_pipe(dir,data,start,num,parameter,iqpnni,scale,lscale,boot,mult,theta,sec,phylo):
    
    sec_list=[int(i) for i in open(sec).readlines()]
    source=len(sec_list)

    name="alg"
    
    print "Make all Directories and Simulate all Sequences"
    if len(scale)>0: scale_list=[i[:-1] for i in open(scale).readlines()]
    if not mult:tree=dir+"model_tree"
    else:
        tree=[]
        for i in range (1,source+1):
            tree.append("%smodel_tree%u.tree"%(dir,i))            
    for i in range (start,num+1):
        if len(scale)>0:
            mysim=simulate(dir,data,i,dir+parameter,tree,scale_list[(i-start)*source:(i-start+1)*source],lscale,theta,sec_list)
        else:
            mysim=simulate(dir,data,i,dir+parameter,tree,seq_scale=lscale,theta=theta,sec_list=sec_list)
        
            
    print "Create supermatrix, single matrices and reconstruct trees"

    for i in range (start,num+1):
        actdir="%ssim_%u/"%(dir,i)
        mysm=Supermatrix(infile=actdir+name,taxdirfile=data+".taxa")
        mysm.create_org_matrices(sections=sec)
        reconstruct(actdir,actdir+name,mysm,iqpnni,boot,phylo) #SM and source trees


options,args=parse_options()
new_pipe(options.dir,options.data,int(options.start),int(options.end),options.seqgen,options.iqpnni,options.scale,float(options.lscale),int(options.boot),options.mult,float(options.theta),options.sec,options.phylo)

